const express = require("express");
const ViteExpress = require("vite-express");
const { PrismaClient } = require("@prisma/client");

const app = express();
app.use(express.json()); 

const prisma = new PrismaClient();

app.use(express.urlencoded({
  extended: true
}));

app.get("/hello", (req, res) => {
 res.send("Hello Vite + React!");
});

ViteExpress.listen(app, 80, () =>
 console.log("Server is listening on port 80...")
);


const staffRoutes = require('./routes/staff.js');
const roomRoutes = require('./routes/room.js');
const maintenanceRoutes = require('./routes/maintenance.js');
const reportRoutes = require('./routes/report.js');

app.use('/staff', staffRoutes);
app.use('/room', roomRoutes);
app.use('/maintenance', maintenanceRoutes);
app.use('/report', reportRoutes);


// Legacy
app.get("/laporan", async (req, res) => {
  try {
    console.log("Mengampil laporan..."); 

    const result = await prisma.laporan_maintenance.findMany({
      where: {
        is_deleted: false
      },
    });

    console.log("Laporan:", result); 

    res.json(result);
  } catch (error) {
    console.error("Error:", error); 

    res.status(500).json({ error: "Error" });
  }
});



app.get("/laporan/:id", async (req, res) => {
  const { id } = req.params;

  try {
    console.log(`Fetching report with id ${id}...`); // Log before fetching the report

    const result = await prisma.laporan_maintenance.findUnique({
      where: {
        id: Number(id),
      },
    });

    if (!result) {
      return res.status(404).json({ error: `No report found with id ${id}` });
    }

    console.log("Report fetched:", result); // Log the fetched report

    res.json(result);
  } catch (error) {
    console.error(`An error occurred while fetching the report with id ${id}:`, error); // Log any errors

    res.status(500).json({ error: `An error occurred while fetching the report with id ${id}` });
  }
});


app.delete("/laporan/:id", async (req, res) => {
  const { id } = req.params;

  try {
    console.log(`Soft deleting report with id ${id}...`); // Log before soft deleting the report

    const result = await prisma.laporan_maintenance.update({
      where: {
        id: Number(id),
      },
      data: {
        is_deleted: true,
      },
    });

    if (!result) {
      return res.status(404).json({ error: `No report found with id ${id}` });
    }

    console.log("Report soft deleted:", result); // Log the soft deleted report

    res.json(result);
  } catch (error) {
    console.error(`An error occurred while soft deleting the report with id ${id}:`, error); // Log any errors

    res.status(500).json({ error: `An error occurred while soft deleting the report with id ${id}` });
  }
});



app.delete("/laporan/hard/:id", async (req, res) => {
  const { id } = req.params;

  try {
    console.log(`Hard deleting report with id ${id}...`); // Log before hard deleting the report

    const report = await prisma.laporan_maintenance.findUnique({
      where: {
        id: Number(id),
      },
    });

    if (!report) {
      return res.status(404).json({ error: `No report found with id ${id}` });
    }

    if (!report.is_deleted) {
      return res.status(400).json({ error: `Report with id ${id} has not been soft deleted` });
    }

    const result = await prisma.laporan_maintenance.delete({
      where: {
        id: Number(id),
      },
    });

    console.log("Report hard deleted:", result); // Log the hard deleted report

    res.json(result);
  } catch (error) {
    console.error(`An error occurred while hard deleting the report with id ${id}:`, error); // Log any errors

    res.status(500).json({ error: `An error occurred while hard deleting the report with id ${id}` });
  }
});




//RADIT START HERE

var jwt = require("jsonwebtoken");
var bcrypt = require("bcrypt");
var  SECRET_KEY = "E9999303D232192560C0DA1B4A1ACB1F94B9E3205F72757530CE16A63B6DFD50" 




app.post("/login", async (req, res) => {

  let usernameinput = req.body.username
  let passwordinput = req.body.password
  const findUser = await prisma.staff.findUnique({
    where : {
      username : usernameinput
    }
  })

  if (!findUser){
    return res.status(401).send("User doesn't exist")
  } else {
    var correctPassword = findUser.password == passwordinput

    if ((!correctPassword)) {
      return res.status(401).send("Wrong credential")
    } else {
      let payload = { id : findUser.id, role : findUser.role};
      const token = jwt.sign(payload, SECRET_KEY, {
        expiresIn: 86400});
      res.status(200).header("authorization", token).send({
        message: "Login successfull",
        accessToken: token,
      });
    }
  }
}) 

// Staff
// GET - semua staf (DENGAN ADMIN AND PASSWORD AND DELETED STAFF)
app.get("/staff/all", verifyUserToken ,async (req, res) => {
  try {
    const staff = await prisma.staff.findMany();
    res.json(staff);
  } catch (error) {
    console.error("Error saat mengambil staf:", error);
    res.status(500).json({ error: "Error saat mengambil staf" });
  }
});


function verifyUserToken(req, res, next) {
  let token =  req.headers.authorization;
  if (!token) {
    return res.status(401).send("Access Denied : No Token");
  } 
  try {
    // token = token.split(" ")[1];
    if (token == null || !token) {
      return res.status(401).send("Access Denied : No Token 2");
    }
    let verifiedUser = jwt.verify(token, SECRET_KEY)
    if (!verifiedUser) {
      return res.status(401).send("Access Denied");
    }
    req.user = verifiedUser;
    next();
  } catch (error) {
    res.status(400).send("Invalid Token")
  }
}

function isAdmin(req, res, next){
  if (req.user.role == "admin"){
    next();
  } else {
    return res.status(401).send("Wrong Privilege")
  }
}

function isStaff(req, res, next){
  if (req.user.role == "staff"){
    next();
  } else {
    return res.status(401).send("Wrong Privilege")
  }
}
